﻿using ApiFrame.Client.Gateway;
using ApiFrame.Framework.Injection;
using ApiFrame.Framework.Interface;
using ApiFrame.Framework.Signature;
using ApiFrame.Framework.Token;
using ApiFrame.Server.Exception;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;

namespace ApiFrame.WebApi.ClientDemo
{
    class Program
    {
        static void Main(string[] args)
        {
            // Service Url
            string serviceUrl = "http://localhost:6360/";

            // Uri version
            string apiVersion = "v2";

            // Configuration
            ApiObjectFactory.RegisterType<IApiSignature, ApiSignature>();
            const string AuthScheme = "HMAC";
            const string AppAccessToken = "d85aa01c-cf92-4f1e-9862-638fa4ae37b4";
            const string AppSecretToken = "06fb3823-b8cc-462b-b855-f655d49eb3e2";
                        
            // Authentication -> Sing-in demo - START
            Console.WriteLine("\n Authentication -> Sign-in demo - START");
            Console.Write("\n\n Enter the Username:");
            string userName = Console.ReadLine();
            Console.Write(" Enter the Password:");
            string password = Console.ReadLine();

            string postData = string.Format("Username={0}&Password={1}", userName, password);

            ApiRequestToken requestToken = new ApiRequestToken()
            {
                Verb = HttpMethod.Post,
                RelativeUrl = string.Format("{0}/demo/signin", apiVersion),
                AccessToken = AppAccessToken,
                SecretToken = AppSecretToken,
                AuthScheme = AuthScheme,
                Content = postData
            };

            ApiClientGateway apiClientGateway = new ApiClientGateway(serviceUrl);
            AccessModel signInResult = null;
            try
            {
                signInResult = apiClientGateway.Execute<AccessModel>(requestToken);

                if (signInResult != null)
                {
                    Console.WriteLine("\n Response from server");
                    Console.WriteLine("\n AccessToken:" + signInResult.AccessToken);
                    Console.WriteLine(" SecretToken:" + signInResult.SecretToken);
                    Console.WriteLine("\n Successfully Authenticated! This is ApiAuthentication Test");
                }
            }
            catch (ApiRequestException ex)
            {
                Console.WriteLine(ex.ErrorResponseMessage.
                    Content.ReadAsStringAsync().Result);
            }

            // Sing-in demo - END

            // Authorization -> GetMessage demo - Start

            Console.WriteLine("\n Authorization demo -> GetMessage - START");
            Console.Read();

            if (signInResult != null)
            {
                requestToken = new ApiRequestToken()
                {
                    Verb = HttpMethod.Get,
                    RelativeUrl = string.Format("{0}/demo/GetMessage", apiVersion),
                    AccessToken = signInResult.AccessToken,
                    SecretToken = signInResult.SecretToken,
                    AuthScheme = AuthScheme
                };

                string messageResult = string.Empty;

                try
                {
                    messageResult = apiClientGateway.Execute(requestToken);

                    if (!string.IsNullOrEmpty(messageResult))
                    {
                        Console.WriteLine("\n This is ApiAuthorization test");
                        Console.WriteLine("Server Response :" + messageResult);
                    }
                }
                catch (ApiRequestException ex)
                {
                    Console.WriteLine(ex.ErrorResponseMessage.
                        Content.ReadAsStringAsync().Result);
                }
            }

            Console.Read();
            Console.Read();
            // Authorization -> GetMessage demo - End

        }

        public class AccessModel
        {
            public string AccessToken { get; set; }
            public string SecretToken { get; set; }
        }
    }
}
